//  ********************************************** ADF5355_Nano ***********************************************************
//  ADF5355 and Arduino Nano
//  Based on the sketch "ADF5355_Arduino_LCD_V0" written by Alain Fort F1CJN and Dave Brink (for 64bits routines)jan 3,2018
//   
//  Modifications by PA0RWE:  (29-04-2018 v1.0)
//  - Put some code in separate subs
//  - Replaced the pushbuttons by 2 encoders 
//  - Add a menu for different functions
//    - Memory storage
//    - Stepsize setting
//    - Output selection (-4 trough +5 dBm)
//    - Reference frequency (25 or 10 MHz)
//
//  Modifications by PA0RWE:  1-11-2018 v1.1)
//  - Display error at power output update
//  - Repaired update dBm directly after setting
//  - Repaired reset to actual frequency if not used memory
//  - Added save last used frequency
//
//  Modifications by PA0RWE:  10-02-2019 v1.2)
//  - Added storage of Power and Stepsize
//  - Storage in memory locations by pressing the Menu button for about 2 seconds
//
//  Modifications by PA0RWE:  01-08-2019 v1.3)
//  - X-tal freq was not proper stored after change
//  - stepsize below 1 KHz not correct
//
//  Modifications by PA0RWE:  21-03-2022 v1.4)
//  - Output +5dBm disabled in this configuration
//  - MTLD set to 1 (Output muted when no lock)
//
//  Modifications by PA0RWE:  20-12-2022 v1.5)
//  - Add 100MHz X-tal freq setting on request of Jac, PE1KXH
//  - Changed storage of X-tal freq by Ref pointer
//  - Output +5dBm enabled again
//
//  ******************************************** HARDWARE IMPORTANT********************************************************
//  With an Arduino NANO : uses a resistive divider to reduce the voltage, MOSI (pin 11) to
//  ADF DATA, SCK (pin13) to ADF CLK, Select (PIN 10) to ADF LE
//  Resistive divider 560 Ohm with 1000 Ohm to ground on Arduino pins 11, 13 and 10 to adapt from 5V
//  to 1.8V the digital signals DATA, CLK and LE send by the Arduino.
//  Arduino pin A1 (for lock detection) directly connected to ADF5355 card MUX-OUT.
//  The ADF card is with 6V seperated powered because of the high power consumption (between 200 and 250 mA).
//  ***********************************************************************************************************************
//
//  EEPROM Storage locations
//  ------------------------
//   Mem Rfint FPFD  HERTZ
//    0   0     80    160   MEM locations start (0)
//    1   4     84    164
//    2   8     88    168
//    3   12    92    172
//    4   16    96    176
//    5   20   100    180
//    6   24   104    184
//    7   28   108    188
//    8   32   112    192
//    9   36   116    196   MEM Locations end (9)
//                    200   Stored actual frequency
//                    220   Stored flag (=50)
//                    224   Power
//                    228   Stepsize pointer
//                    232   Ref pointer


#include <LiquidCrystal.h>
#include <EEPROM.h>
#include <SPI.h>

#define ADF5355_LE 10
#define R04 0X36008B84          // DB4=0
#define R04_EN 0X36008B94       // DB4=1
//#define MODE2  1000 

#define encoderLPinA 2          // interrupt pin
#define encoderLPinB A2         // no need to use an interrupt pin here
#define encoderRPinA 3          // interrupt pin
#define encoderRPinB A3         // no need to use an interrupt pin here
#define encoderBtn   A0         // Encoder pushbutton A0

volatile long posL = 0;         // long can hold far more pulses than int; signed to know direction since start.
volatile boolean CWL = true;    // direction if false it is CCWL
volatile long posR = 0;         // long can hold far more pulses than int; signed to know direction since start.
volatile boolean CWR = true;    // direction if false it is CCWR
volatile boolean Left = false;
volatile boolean Right = false;

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

byte line = 0, X2=0;            // ligne afficheur LCD en cours 0 ou 1

// Init settings 13 registers  0        1           2       3        4          5         6           7           8           9         10        11        12
uint32_t registers [13] = {0x200E60, 0xC4EC4E1, 0xC043E82, 0x3, 0x36008B84, 0x800025, 0x35A12076, 0x120000E7, 0x102D0428, 0x5047CC9, 0xC0067A, 0x61300B, 0x1041C}; // 187,5 MHz with a 25 MHz Reference Clock

int address;
unsigned int i = 0;

double StepSize[10] = {1000000, 100000, 10000, 1000, 100, 10, 1, 100, 10, 1};
int CursPos[10] = {1, 3, 4, 5, 7, 8, 9, 11, 12, 13};   // Stepsize cursor position
const char OutdB[ ][7] = {" -4dBm", " -1dBm", " +2dBm", " +5dBm"};
const char Step[ ][10] = {"1GHz  ", "100MHz", "10MHz ", "1MHZ  ", "100KHz", "10KHz ", "1KHz  ", "100Hz ", "10Hz  ", "1Hz   "};
unsigned long RefS[3] = {10, 25, 100};  // Ref frequencies
unsigned long oldRef = 0;     // Old Ref value
int RefP = 1;                 // Reffreq pointer
int dBmP = 1;                 // Output dBm pointer
int olddBmP = 0;              // Old dBm setting
int StepP = 4;                // Stepsize pointer for StepSize[]
int Menu = 0;                 // Menu items 0=normal tuning, 1=mempos, 2=stepsize, 3=output, 4=reffreq
int menumode = 0;             // In menu mode or not
int btnflag = 0;              // Flag when button is pressed
int memoire = 0;              // Memory position
int storeflag = 0;            // Flag that actual freq is saved

unsigned long startTime;
unsigned long endTime;
unsigned long duration;
byte timerRunning;
unsigned long currentMillis = millis();
unsigned long startMillis = 0;
boolean  actupdt = false;

double RFout, REFin, OutputChannelSpacing;
unsigned long RFint, RFintold, RFintstore, RFcalc, RFOUT, TEST;
long RF_Divider, HERTZ, HERTZold, HERTZstore; 
unsigned long FVCO[]={0,600000000};     // en dizaine de Hz*
unsigned long FPFD[]={0,0};             // en MHz
unsigned long FVCOTEMP[]={0,0};         // en dizaine de Hz*
unsigned long FPFDTEMP[]={0,0};
unsigned long N10E6[]={0,1000000};
unsigned long N10E3[]={0,1000};
unsigned long INT[]={0x0,0x0};
unsigned long N[]={0x0,0x0};
unsigned long NX[]={0x0,0x0};
unsigned long FRAC1[]={0x0,0x0};
unsigned long FRAC11[]={0x0,0x0};
unsigned long FRAC2[]={0x0,0x0};
unsigned long HZ[]={0x0,0x0};
unsigned long FRAC1exact[]={0x0,0x0};
unsigned long MOD1[]={0,16777216};
unsigned long MOD2[]={0,1000};

// unsigned int long reg0, reg1;
unsigned int long FRAC1R, FRAC2R, INTR, RF_Select, I_BLEED_N, RFB, MTLD;

//**************  64 bits routines written by Dave Brink *************************************
unsigned long zero64[]={0,0}; //just for comparisons sake

void init64(unsigned long  an[], unsigned long bigPart, unsigned long littlePart ){
 an[0]=bigPart;
 an[1]=littlePart;
}

//left shift 64 bit "number"
void shl64(unsigned long  an[]){
an[0] <<= 1;
if(an[1] & 0x80000000)
  an[0]++;
an[1] <<= 1;
}

//right shift 64 bit "number"
void shr64(unsigned long  an[]){
an[1] >>= 1;
if(an[0] & 0x1)
  an[1]+=0x80000000;
an[0] >>= 1;
}

//add ann to an
void add64(unsigned long  an[], unsigned long  ann[]){
 an[0]+=ann[0];
 if(an[1] + ann[1] < ann[1])
   an[0]++;
 an[1]+=ann[1];
}

//subtract ann from an
void sub64(unsigned long  an[], unsigned long  ann[]){
 an[0]-=ann[0];
 if(an[1] < ann[1]){
   an[0]--;
 }
 an[1]-= ann[1];
}

//true if an == ann
boolean eq64(unsigned long  an[], unsigned long  ann[]){
 return (an[0]==ann[0]) && (an[1]==ann[1]);
}

//true if an < ann
boolean lt64(unsigned long  an[], unsigned long  ann[]){
 if(an[0]>ann[0]) return false;
 return (an[0]<ann[0]) || (an[1]<ann[1]);
}

//divide num by den
void div64(unsigned long num[], unsigned long den[]){
 unsigned long quot[2];
 unsigned long qbit[2];
 unsigned long tmp[2];
 init64(quot,0,0);
 init64(qbit,0,1);

 if (eq64(num, zero64)) {  //numerator 0, call it 0
   init64(num,0,0);
   return;            
 }

 if (eq64(den, zero64)) { //numerator not zero, denominator 0, infinity in my book.
   init64(num,0xffffffff,0xffffffff);
   return;            
 }

 init64(tmp,0x80000000,0);
 while(lt64(den,tmp)){
   shl64(den);
   shl64(qbit);
 }
 
 while(!eq64(qbit,zero64)){
   if(lt64(den,num) || eq64(den,num)){
     sub64(num,den);
     add64(quot,qbit);
   }
   shr64(den);
   shr64(qbit);
 }

 //remainder now in num, but using it to return quotient for now  
 init64(num,quot[0],quot[1]);
}

//multiply num by den
void mul64(unsigned long an[], unsigned long ann[]){
 unsigned long p[2] = {0,0};
 unsigned long y[2] = {ann[0], ann[1]};
 while(!eq64(y,zero64)) {
   if(y[1] & 1)
     add64(p,an);
   shl64(an);
   shr64(y);
 }
 init64(an,p[0],p[1]);
}

//**************************** Testprint variable f64 ****************************************
void prt(unsigned long  an[]){
Serial.print(an[0],DEC);
Serial.print(" ");
Serial.println(an[1],DEC);
}

//**************************** ENCODER ROUTINES **********************************************
void doEncoderL() // update Left encoder 100MHz / Menu select
{
  // determine direction
  CWL = (digitalRead(encoderLPinB) != digitalRead(encoderLPinA)); 
  
  if (CWL) {
    if (menumode == 0) {
      RFint = RFint + 100000 ;
      if (RFint > 13600000) RFint = RFintold;
    }
    else  {
      Menu +=1;
      if (Menu > 4) Menu=4;
    }
  }
  else {
    if (menumode == 0) {
      RFint = RFint - 100000 ;
      if (RFint < 54000 | RFint > 13600000) RFint = RFintold;
    }
    else  {
      Menu -=1;
      if (Menu < 0) Menu=0;
    }
  }
}

void doEncoderR()   // update Right encoder Tuning / Set-up Menu
{
  // determine direction
  CWR = (digitalRead(encoderRPinB) != digitalRead(encoderRPinA)); 

  if (CWR) {
    switch (Menu) {
    case 0:
      if (StepP < 7)  RFint = RFint + StepSize[StepP];
      else HERTZ = HERTZ + StepSize[StepP];
      if (HERTZ >= 1000) {HERTZ=HERTZ-1000; RFint=RFint+1;} 
      if (RFint > 13600000)RFint = RFintold;
    break;
    case 1:                           // Memory
      memoire += 1;
      if (memoire > 9) memoire = 9;
    break;
    case 2:                           // Stepsize
      StepP += 1;
      if (StepP > 9) StepP = 9;
//      lcd.setCursor(CursPos[StepP], 0);
    break;
    case 3:                           // Output power 
      dBmP += 1;
      if (dBmP > 3) dBmP = 3;
    break;
    case 4:                           // Ref frequency
      oldRef = FPFD[1];               // Save old Ref
      RefP += 1;
      if (RefP > 2) RefP = 2;
      FPFD[1] = RefS[RefP];
      EEPROM.write(232, RefP);        // Write Ref pointer    
    break;
    }
  }
  else {
    switch (Menu) {
    case 0:
      if (StepP < 7)  RFint = RFint - StepSize[StepP];
      else HERTZ = HERTZ - StepSize[StepP];
      if (HERTZ < 0) {HERTZ=1000+HERTZ; RFint=RFint-1;} 
      if (RFint < 54000) RFint = RFintold;
    break;
    case 1:
      memoire -= 1;
      if (memoire < 0) memoire = 0;
     break;
    case 2:
      StepP -= 1;
      if (StepP < 0) StepP = 0;
//      lcd.setCursor(CursPos[StepP], 0);
    break;
    case 3:
      dBmP -= 1;
      if (dBmP < 0) dBmP = 0;
    break;
    case 4:
      oldRef = FPFD[1];            // Save old Ref
      RefP -=1;
      if (RefP < 0) RefP = 0;
      FPFD[1] = RefS[RefP];
      EEPROM.write(232, RefP);        // Write Ref pointer   
    break;
    }
  }
}

//*********************** Read the encoderbutton with debouncing *****************************
boolean get_button()
{
  if (!digitalRead(encoderBtn))            // Menu button
  {
    delay(20);
    startTime = millis();                 // Start counting
    if (!digitalRead(encoderBtn))         // Still pressed
    {
      while (!digitalRead(encoderBtn));   // Wait until released
      btnflag = true;      
    }
    duration = millis() - startTime;      // Total time pressed
  }
  if (btnflag) return 1;
  else return 0;
}


//*********************************** Output to LCD ******************************************
void printAll()
{
// Line 1
  lcd.setCursor(0,0);                         // line 1 pos 0
  RFcalc=(RFint/1000000);
  if (RFcalc<10) lcd.print(" ");
  if (RFcalc==0) lcd.print(" ");
  if (RFcalc>0){
    lcd.print(RFcalc);                        // Print GHz
    lcd.print(".");
  }
  else lcd.print(" ");
   
  RFcalc=RFint-((RFint/1000000)*1000000);
  if (RFcalc < 100000) lcd.print("0");
  if (RFcalc < 10000)  lcd.print("0"); 
  lcd.print(RFcalc/1000); lcd.print(".");     // Print MHz
  
  RFcalc=RFint-((RFint/1000)*1000);
  if (RFcalc<100)lcd.print("0");
  if (RFcalc<10)lcd.print("0");
  lcd.print(RFcalc);                          // Print KHz
  lcd.print(".");
  RFcalc=HERTZ-((HERTZ/1000)*1000);
  if (RFcalc<100)lcd.print("0");
  if (RFcalc<10)lcd.print("0");
  lcd.print(HERTZ);                           // Print Hz
  if (menumode == 0) lcd.print("Hz");
  else lcd.print(" M");                       // in menumode "Hz" = " M"

// Line 2
  if (menumode==0) {
    lcd.setCursor(0,1);                       // line 2 pos 0
    lcd.print("Mem=");
    lcd.print(memoire,DEC);
    delay(100);

    lcd.print(OutdB[dBmP]);                   // Output power
    if  ((digitalRead(A1)==1))lcd.print(" L");
    else lcd.print(" X");
    lcd.print(FPFD[1],DEC);                   // X-tal frequency
    if (FPFD[1] < 100) {
      lcd.setCursor(15,1);                    // space on pos 15
      lcd.print(" ");
    }  
  }
  else {
    lcd.setCursor(0,1);                       // line 2 pos 0
    switch (Menu) { 
    case 1:
      lcd.print("    Memory= ");
      lcd.print(memoire,DEC);                 // Memory position
      lcd.print("   ");
    break;
    case 2:
      lcd.print("Stepsize: ");
      lcd.print(Step[StepP]);                 // Step position
    break;
    case 3:
      lcd.print(" Output: ");
      lcd.print(OutdB[dBmP]);                 // Output power
      lcd.print(" ");
    break;
    case 4:
    lcd.print("  Ref freq.= ");
    lcd.print(FPFD[1],DEC);                   // X-tal frequency
    if (FPFD[1] < 100) {
      lcd.setCursor(15,1);                    // space on pos 15
      lcd.print(" ");
    }
    break;    
    }
  }
  lcd.setCursor(CursPos[StepP], 0);           // display actual stepsize with cursor
}

void WriteRegister32(const uint32_t value)    //Programme un registre 32bits
{
  digitalWrite(ADF5355_LE, LOW);
  for (int i = 3; i >= 0; i--)                // boucle sur 4 x 8bits
  SPI.transfer((value >> 8 * i) & 0xFF);      // décalage, masquage de l'octet et envoi via SPI
  digitalWrite(ADF5355_LE, HIGH);
  digitalWrite(ADF5355_LE, LOW);
}

void SetADF5355()                             // Programme tous les registres de l'ADF5355
{ for (int i = 13; i >= 0; i--)               // programmation ADF5355 en commencant par R12
    WriteRegister32(registers[i]);
}


// *************** Write long (32bits) to EEPROM  in 4 stappen van 8 bits **************
void EEPROMWritelong(int address, long value)
{
  //Decomposition du long (32bits) en 4 bytes
  //trois = MSB -> quatre = lsb
  byte quatre = (value & 0xFF);
  byte trois = ((value >> 8) & 0xFF);
  byte deux = ((value >> 16) & 0xFF);
  byte un = ((value >> 24) & 0xFF);

  //Ecrit 4 bytes dans la memoire EEPROM
  EEPROM.write(address, quatre);
  EEPROM.write(address + 1, trois);
  EEPROM.write(address + 2, deux);
  EEPROM.write(address + 3, un);
}

// *************** Read long (32bits) van EEPROM in 4 stappen van 8 bits **************
long EEPROMReadlong(long address)
{
  //Read the 4 bytes from the eeprom memory.
  long quatre = EEPROM.read(address);
  long trois = EEPROM.read(address + 1);
  long deux = EEPROM.read(address + 2);
  long un = EEPROM.read(address + 3);

  //Retourne le long(32bits) en utilisant le shift de 0, 8, 16 et 24 bits et des masques
  return ((quatre << 0) & 0xFF) + ((trois << 8) & 0xFFFF) + ((deux << 16) & 0xFFFFFF) + ((un << 24) & 0xFFFFFFFF);
}

//************************* Calculate Registers, update ADF5355 and LCD ********************************************************
void Upd_ADF5355()
{
  if (RFOUT >= 6800000)  {RF_Divider = 1;  RF_Select = 0; X2=1; RFB=0;} // RFOUT = Frequentie in KHz  et mettre la sortie X2 à 1
  if (RFOUT  < 6800000)  {RF_Divider = 1;  RF_Select = 0;};
  if (RFOUT  < 3400000)  {RF_Divider = 2;  RF_Select = 1;};
  if (RFOUT  < 1700000)  {RF_Divider = 4;  RF_Select = 2;};
  if (RFOUT  < 850000)   {RF_Divider = 8;  RF_Select = 3;};
  if (RFOUT  < 425000)   {RF_Divider = 16; RF_Select = 4;};
  if (RFOUT  < 212500)   {RF_Divider = 32; RF_Select = 5;};
  if (RFOUT  < 106250)   {RF_Divider = 64; RF_Select = 6;};
    
  FVCO[1]=RFint * RF_Divider;
  FPFDTEMP[0]=FPFD[0]; FPFDTEMP[1]=FPFD[1];
  FVCOTEMP[0]=FVCO[0]; FVCOTEMP[1]=FVCO[1];
 
  init64(N,0,10000);
  mul64(FVCO,N);

  init64(N,0,2);                // Prepaire X2=2 if 6.8GHZ < RFOUT < 13.6GHz
  if (X2==1){div64(FVCO,N);};   // Devide by 2 bij 6.8GHZ < RFOUT < 13.6GHz so that FVCO stays between 3.4 et 6.8GHz
 
  init64(HZ,0,RF_Divider*10*HERTZ); 
  init64(N,0,2);                // new init to N=2 because div64 modifies N
  if (X2==1){div64(HZ,N);};
  add64(FVCO,HZ);               // addition of µHz

  init64(N,0,10000);            // FVCO * 10000
  mul64(FVCO,N);
  div64(FVCO,FPFD);             // FVCO / FPFD
  N[0]=FVCO[0];N[1]=FVCO[1];
  INT[0]=N[0];INT[1]=N[1];      // INT = FVCO / FPFD
  init64(NX,0,10E6);            
  div64(INT,NX);
  init64(NX,0,10000);
  div64(INT,NX);
  INTR=(INT[1]);
  init64(NX,0,10E4);
  mul64(INT,NX);
  init64(NX,0,10E5);
  mul64(INT,NX);
  sub64(N,INT);
  mul64(N,MOD1);
  FRAC1exact[0]=N[0];FRAC1exact[1]=N[1];
  init64(NX,0,10E7);
  div64(N,NX);
  init64(NX,0,1000);
  div64(N,NX);
  FRAC1R=N[1];
  FRAC11[0]=N[0];FRAC11[1]=N[1];
  init64(N10E6,0,1000000);      //prt(N10E6);
  mul64(FRAC11,N10E6);
  init64(N10E3,0,100000);       //prt(N10E3);
  mul64(FRAC11,N10E3);
  sub64(FRAC1exact,FRAC11);
  init64(NX,0,10000);
  div64(FRAC1exact,NX);
  init64(NX,0,1000);
  div64(FRAC1exact,NX);
  init64(NX,0,5);               // to achieve a correct rounding
  add64(FRAC1exact,NX);
  init64(NX,0,10); 
  div64(FRAC1exact,NX);
  FRAC2R=FRAC1exact[1];
  FPFD[0]=FPFDTEMP[0]; FPFD[1]=FPFDTEMP[1];
  FVCO[0]=FVCOTEMP[0]; FVCO[1]=FVCOTEMP[1];
  //I_BLEED_N=7;
 
  RFintold = RFint;
  HERTZold = HERTZ;
  olddBmP  = dBmP;

  startMillis = millis();   // Reset actual freq store timer
  actupdt = false;

// ************************ Write ADF5355 registers ***************************************
//
  I_BLEED_N=10;     // Charge pump bleed current 
  MTLD = 1;
  
  Serial.println("");
  if (RFB==1) {Serial.println ("RFB = OFF");} else {Serial.println("RFB = ON");}         
  Serial.print("RF_Select=");  Serial.println(RF_Select,DEC);
  Serial.print("INT= ");       Serial.println(INTR);
  Serial.print("MOD2 = ");     Serial.println(MOD2[1]);
  Serial.print("FRAC1 = ");    Serial.println(FRAC1R);
  Serial.print("FRAC2 = ");    Serial.println(FRAC2R);

// Sequence according data sheet

  WriteRegister32(registers[10]);                            // R10
  WriteRegister32((RF_Select<<21)|(I_BLEED_N<<13)|(MTLD<<11)|(RFB<<10)|(0x35000046)|(dBmP<<4));    //R6
  WriteRegister32(4|R04_EN);                                 // R4  DB4=1
  WriteRegister32(2|(1000<<4)|(FRAC2R<<18));                 // R2  MOD2=1000
  WriteRegister32(1|(FRAC1R<<4));                            // R1
  WriteRegister32(0|INTR<<4);                                // R0   
  WriteRegister32(R04);                                      // R4  DB4=0
  delay(3);                                                  // Wait 3 ms to stabilize VCO
  WriteRegister32(0|0X200000|(INTR<<4));                     // R0    
     
//  if (FPFD[1] > 75) {
//    WriteRegister32(4|R04_EN);                                 // R4  DB4=1
//    WriteRegister32(2|(1000<<4)|(FRAC2R<<18));                 // R2
//    WriteRegister32(1|(FRAC1R<<4));                            // R1
//    WriteRegister32(0|INTR<<4);                                // R0   
//  }

//  ***** T E S T *******
  for (int i = 6; i >= 0; i--) {              // programmation ADF5355 
    Serial.print("TESTR-");
    Serial.print(i);
    Serial.print("-");
    Serial.println(registers[i],HEX);
  }
//  ***** T E S T *******
  
  printAll();  // Affichage LCD
  Serial.println("Update"); 
}

//***************************************************************************************
//                                   S E T U P 
//***************************************************************************************
void setup() {
 
  Serial.begin (115200);    //  Serial to the PC via Arduino "Serial Monitor"  at 115200
  lcd.begin(16, 2);         // two 16 characters lines
  lcd.display();
  lcd.setCursor(0, 0);
  lcd.print("     PA0RWE     ");
  lcd.setCursor(0, 1);
  lcd.print("ADF5355 VFO V1.5");
  delay(2000);
  
  // CONFIG IRQ
  pinMode(encoderLPinA, INPUT);
  attachInterrupt(0, doEncoderL, FALLING);
  pinMode(encoderRPinA, INPUT);
  attachInterrupt(1, doEncoderR, FALLING);

  pinMode(encoderBtn, INPUT_PULLUP);    // Encoder switch
  pinMode(A1, INPUT);                   // PIN A1 input MUX (Lock detect)
  pinMode(ADF5355_LE, OUTPUT);          // Setup pins
  digitalWrite(ADF5355_LE, HIGH);
  
  SPI.begin();                          // Init SPI bus
  SPI.setDataMode(SPI_MODE0);           // CPHA = 0 et Clock positive
  SPI.setBitOrder(MSBFIRST);            // poids forts en tête

// Read EEPROM Locations
  if (EEPROM.read(220)==50){            // Read write flag
     RFint = EEPROMReadlong(40);        // Read RFint stored actual frequency
     HERTZ = EEPROMReadlong(200);       // Read HERTZ 
     if ((HERTZ >= 1000)||(HERTZ<0)) HERTZ=0; 
     dBmP  = EEPROM.read(224);          // Read Power setting
     StepP = EEPROM.read(228);          // Read Stepsize pointer
     RefP  = EEPROM.read(232);          // Read x-tal frequency pointer
  }
  else {RefP=1; RFint=144500; HERTZ=0; dBmP=2; StepP=2;}        // Init for the first time
  
  FPFD[1] = RefS[RefP];         // Setup Ref frequency
  RFintold=187500;              // pour que RFintold soit different de RFout lors de l'init //////Avoir
  RFOUT = 187500 ;              // fréquence de sortie en khz
  OutputChannelSpacing = 1;     // Pas de fréquence = 1kHz

  address=0;
  lcd.cursor();
  printAll(); delay(500);
  SetADF5355();                 // Write registers and set clock to 25 MHz

//  currentMillis = millis();
  startMillis = millis();       // Reset actual freq store timer
  actupdt = true;

  RFintold=123456;
  RFB=1;                        // RFB output Off
} // Fin setup

//***************************************************************************************
//                                   L O O P 
//***************************************************************************************
void loop()
{
//  RFint = 11300000;   // Freq in KHz
//  HERTZ = 000;        // Freq in Hz (max 999)
  
  RFOUT=RFint; RFB=1; X2=0;                                   // RFB output off

  if ((RFint != RFintold)||(HERTZ!=HERTZold)) Upd_ADF5355();  // Update ADF when freq is changed
  if (dBmP != olddBmP) Upd_ADF5355();                         // Update ADF when Output is changed

// Store actual freq once per 5 seceonds after frequency change
  if((millis() - startMillis) > 5000 && actupdt == false) {
    EEPROMWritelong(40,RFint);                // Write RFint en EEPROM à adresse (memoire*4)
    EEPROMWritelong(200,HERTZ);               // Write HERTZ en EEPROM à adresse (memoire*4)+160
    EEPROM.write(220,50);                     // Write flag en EEPROM à adresse 220 
    EEPROM.write(224, dBmP);                  // Write Power setting   
    EEPROM.write(228, StepP);                 // Write Stepsize pointer     
    EEPROM.write(232, RefP);                  // Write Ref pointer    
    startMillis = millis();                   // Reset actual freq store timer   
    actupdt = true;
    Serial.println("Act freq and settings stored"); 
  }

  if (get_button()) {
    if (btnflag) {
      if (storeflag == 0) {
        RFintstore = RFint;                                   // Save actual frequency
        HERTZstore = HERTZ;                                   // for storing
        storeflag = 1;
      }
      
      if (menumode == 1 && Menu == 1 && duration > 1000) {    // Store freq to memory
        lcd.setCursor(0,1);                                   // line 2 pos 0
        lcd.print("Frequency stored");
        EEPROMWritelong(memoire*4,RFintstore);                // Write RFint en EEPROM à adresse (memoire*4)
        EEPROMWritelong(((memoire*4)+160),HERTZstore);        // Write HERTZ en EEPROM à adresse (memoire*4)+160
        EEPROM.write(220,50);                                 // Write flag en EEPROM à adresse 220
        RFint = RFintstore;                                   // Return stored frequency
        HERTZ = HERTZstore;                                   // after storing
        storeflag = 0;
        delay(3000);
      }

      if (menumode==0) menumode=1; else menumode=0;
      if (menumode==1) Menu=1; else Menu=0;
      printAll();
      btnflag = 0;
      }
  } // end get_button

  if (menumode==0) storeflag = 0;                             // Reset storeflag if no freq store        
  
  if (menumode == 1 && Menu == 1) {                           // Read memory in memory mode
    RFint = EEPROMReadlong(memoire*4);                        // Read EEPROM based on memoire
    if (RFint>13600000) RFint=13600000; 
    HERTZ=EEPROMReadlong((memoire*4)+160);                    // Read HERTZ from EEPROM
    if ((HERTZ >= 1000)|| (HERTZ<0)) HERTZ=0; 
    RefP = EEPROM.read(232);                                  // Read x-tal frequency pointer
    FPFD[1] = RefS[RefP];       
  }

  if (menumode == 1 && Menu != 1) {                           // Reset freq if not used for store
    RFint = RFintstore;                                       // and back in normal mode
    HERTZ = HERTZstore;
  }
  
  if (menumode==1) printAll();                                // Display in Menu mode
}
